@@ -229,7 +229,7 @@ static int get_control(snd_ctl_t *handle, snd_ctl_elem_id_t *id, snd_config_t *t
 {
 	snd_ctl_elem_value_t *ctl;
 	snd_ctl_elem_info_t *info;
-	snd_config_t *control, *comment, *item, *value;
+	snd_config_t *control, *comment, *item = NULL, *value;
 	const char *s;
 	char buf[256];
 	unsigned int idx;
@@ -655,13 +655,16 @@ static long config_iface(snd_config_t *n)
 	const char *str;
 	switch (snd_config_get_type(n)) {
 	case SND_CONFIG_TYPE_INTEGER:
-		snd_config_get_integer(n, &i);
+		if (snd_config_get_integer(n, &i) < 0)
+			return -1;
 		return i;
 	case SND_CONFIG_TYPE_INTEGER64:
-		snd_config_get_integer64(n, &li);
+		if (snd_config_get_integer64(n, &li) < 0)
+			return -1;
 		return li;
 	case SND_CONFIG_TYPE_STRING:
-		snd_config_get_string(n, &str);
+		if (snd_config_get_string(n, &str) < 0)
+			return -1;
 		break;
 	default:
 		return -1;
@@ -681,17 +684,20 @@ static int config_bool(snd_config_t *n, int doit)
 
 	switch (snd_config_get_type(n)) {
 	case SND_CONFIG_TYPE_INTEGER:
-		snd_config_get_integer(n, &val);
+		if (snd_config_get_integer(n, &val) < 0)
+			return -1;
 		if (val < 0 || val > 1)
 			return -1;
 		return val;
 	case SND_CONFIG_TYPE_INTEGER64:
-		snd_config_get_integer64(n, &lval);
+		if (snd_config_get_integer64(n, &lval) < 0)
+			return -1;
 		if (lval < 0 || lval > 1)
 			return -1;
 		return (int) lval;
 	case SND_CONFIG_TYPE_STRING:
-		snd_config_get_string(n, &str);
+		if (snd_config_get_string(n, &str) < 0)
+			return -1;
 		break;
 	case SND_CONFIG_TYPE_COMPOUND:
 		if (!force_restore || !doit)
@@ -718,13 +724,16 @@ static int config_enumerated(snd_config_t *n, snd_ctl_t *handle,
 
 	switch (snd_config_get_type(n)) {
 	case SND_CONFIG_TYPE_INTEGER:
-		snd_config_get_integer(n, &val);
+		if (snd_config_get_integer(n, &val) < 0)
+			return -1;
 		return val;
 	case SND_CONFIG_TYPE_INTEGER64:
-		snd_config_get_integer64(n, &lval);
+		if (snd_config_get_integer64(n, &lval) < 0)
+			return -1;
 		return (int) lval;
 	case SND_CONFIG_TYPE_STRING:
-		snd_config_get_string(n, &str);
+		if (snd_config_get_string(n, &str) < 0)
+			return -1;
 		break;
 	case SND_CONFIG_TYPE_COMPOUND:
 		if (!force_restore || !doit)
@@ -1247,6 +1256,8 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 		}
 		if (strcmp(fld, "iface") == 0) {
 			iface = (snd_ctl_elem_iface_t)config_iface(n);
+			if (iface < 0)
+				return -EINVAL;
 			continue;
 		}
 		if (strcmp(fld, "device") == 0) {
@@ -1254,7 +1265,8 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 				cerror(doit, "control.%d.%s is invalid", numid, fld);
 				return -EINVAL;
 			}
-			snd_config_get_integer(n, &device);
+			if (snd_config_get_integer(n, &device) < 0)
+				return -EINVAL;
 			continue;
 		}
 		if (strcmp(fld, "subdevice") == 0) {
@@ -1262,7 +1274,8 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 				cerror(doit, "control.%d.%s is invalid", numid, fld);
 				return -EINVAL;
 			}
-			snd_config_get_integer(n, &subdevice);
+			if (snd_config_get_integer(n, &subdevice) < 0)
+				return -EINVAL;
 			continue;
 		}
 		if (strcmp(fld, "name") == 0) {
@@ -1270,7 +1283,8 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 				cerror(doit, "control.%d.%s is invalid", numid, fld);
 				return -EINVAL;
 			}
-			snd_config_get_string(n, &name);
+			if (snd_config_get_string(n, &name) < 0)
+				return -EINVAL;
 			continue;
 		}
 		if (strcmp(fld, "index") == 0) {
@@ -1278,7 +1292,8 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 				cerror(doit, "control.%d.%s is invalid", numid, fld);
 				return -EINVAL;
 			}
-			snd_config_get_integer(n, &index);
+			if (snd_config_get_integer(n, &index) < 0)
+				return -EINVAL;
 			continue;
 		}
 		if (strcmp(fld, "value") == 0) {
@@ -1627,6 +1642,7 @@ int load_state(const char *cfgdir, const char *file,
 	snd_config_t *config;
 	const char *cardname1;
 
+	config = NULL;
 	err = load_configuration(file, &config, &open_failed);
 	if (err < 0 && !open_failed)
 		return err;
@@ -1676,7 +1692,8 @@ int load_state(const char *cfgdir, const char *file,
 	}
 	err = finalerr ? finalerr : snd_card_iterator_error(&iter);
 out:
-	snd_config_delete(config);
+	if (config)
+		snd_config_delete(config);
 	snd_config_update_free_global();
 	return err;
 }
